#!/bin/bash

# 10/05/2014, author: Maxim Suhanov

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 2 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

srcdev="$1"
dstdir="$2"
fstype="$3"
mountpoint="$4"
if [ -z "$srcdev" ] || [ -z "$dstdir" ] || [ -z "$fstype" ] || [ -z "$mountpoint" ]; then
	dialog --msgbox "Не выбран накопитель, или не задан путь к файлу для записи данных, или не задан тип файловой системы, или не задана точка подключения файловой системы!" 12 60
	exit 5
fi

log_filename='dc3dd.log'
logfile="$dstdir$log_filename"
filename='image.00000'

# Run dc3dd
clear
echo 'Запуск копирования. Для отмены копирования нажмите Ctrl+C.'
echo ''
if [ "$fstype" == "vfat" ]; then # Segment the output
	dc3dd if="$srcdev" ofs="$dstdir$filename" ofsz=4294967294c log="$logfile"
else # Output into one file
	dc3dd if="$srcdev" of="$dstdir$filename" log="$logfile"
fi

# Examine the log file
grep -q '^dc3dd completed at ' "$logfile"
if [ $? == 0 ]; then  # dc3dd successfully completed
	dialog --msgbox "Копирование успешно завершено!" 12 60
	exec ./255-finish.sh "$mountpoint" # Go to next step
else # dc3dd failed
	question=`echo -e "Копирование было прервано из-за ошибки или из-за его отмены пользователем! Более подробные сведения можно найти в журнальном файле по адресу \"$logfile\".\n\nОтобразить содержимое указанного файла?"`
	dialog --visit-items --cr-wrap --yes-label "Да" --no-label "Нет" --yesno "$question" 12 65

	ec="$?"
	# What did user select?
	case $ec in
		255) exit 255;; # Error or "ESC" key pressed
		0) # "YES" button pressed, display the log
			dialog --exit-label "Вернуться" --textbox "$logfile" 60 75
			exec ./255-finish.sh "$mountpoint"
		;;
		1) exec ./255-finish.sh "$mountpoint";; # "NO" button pressed, go to next step
	esac
fi


