#!/bin/bash

# 10/05/2014, author: Maxim Suhanov

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 2 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

srcdev="$1"
dstdev="$2"
descrfile="$3"
dstdir=''

if [ "$ALTQUIRE_DST" != "/dev/$dstdev" ]; then # Double-check
	dialog --msgbox "Ошибка: \"/dev/$dstdev\" != \"$ALTQUIRE_DST\"!" 12 60
	exit 6
fi

function mountdev_and_prepare()
{
	umount "/dev/$1" 2> /dev/null
	grep -q "^/dev/$1 " /proc/mounts

	ec="$?"
	if [ $ec == 0 ]; then # Already mounted
		dialog --msgbox "Файловая система \"$1\" уже подключена! Перед копированием необходимо произвести отключение указанной файловой системы." 12 60
		exec $0 "$srcdev" "$dstdev" "$descrfile"
	fi

	mountpoint=`mktemp -d 2> /dev/null`
	mount -o rw "/dev/$1" "$mountpoint"

	ec="$?"
	if [ $ec != 0 ]; then # Cannot mount
		rmdir "$mountpoint" 2> /dev/null
		dialog --msgbox "Невозможно подключить выбранную файловую систему! Вероятно, выбранная файловая система повреждена и требует восстановления." 12 60
		exec $0 "$srcdev" "$dstdev" "$descrfile"
	fi

	src_size=`blockdev --getsize64 "/dev/$srcdev" 2> /dev/null || echo 0`
	avail_space_1024=`df -Pk "$mountpoint/." | grep "$mountpoint\$" | awk '{ print $4 }'`
	cant_do_it=$(($src_size-$avail_space_1024*1024))
	if [ $cant_do_it -gt 0 ]; then # Not enough free space in the file system
		umount "$mountpoint" 2> /dev/null
		rmdir "$mountpoint" 2> /dev/null
		dialog --msgbox "В выбранной файловой системе недостаточно свободного пространства (не хватает, байт: $cant_do_it)! Выберите другую файловую систему." 12 60
		exec $0 "$srcdev" "$dstdev" "$descrfile"

	fi

	currtime=`date '+%H-%M-%S_%Y-%m-%d'`
	mkdir "$mountpoint/$currtime" 2> /dev/null

	ec="$?"
	if [ $ec != 0 ]; then # Cannot create directory
		umount "$mountpoint" 2> /dev/null
		rmdir "$mountpoint" 2> /dev/null
		dialog --msgbox "Невозможно создать в выбранной файловой системе директорию с именем \"$currtime\" для сохранения копируемых данных!" 12 60
		exec $0 "$srcdev" "$dstdev" "$descrfile"
	fi

	cp -f "$descrfile" "$mountpoint/$currtime/information.txt" 2> /dev/null

	ec="$?"
	if [ $ec != 0 ]; then # Cannot move file
		umount "$mountpoint" 2> /dev/null
		rmdir "$mountpoint" 2> /dev/null
		dialog --msgbox "Невозможно переместить в выбранную файловую систему файл с описанием обстоятельств копирования данных!" 12 60
		exec $0 "$srcdev" "$dstdev" "$descrfile"
	fi

	dstdir="$mountpoint/$currtime/"
}

if [ -z "$srcdev" ] || [ -z "$dstdev" ] || [ -z "$descrfile" ] || [ ! -r "$descrfile" ]; then
	dialog --msgbox "Не выбраны накопители или не задан файл с описанием обстоятельств копирования данных!" 12 60
	exit 5
fi

devcount=0
menulist=''
lastdev=''
while read line; do # List available file systems
	dev=`echo "$line" | cut -d ' ' -f 1`
	fstype=`echo "$line" | cut -d ' ' -f 2`
	if [ "$fstype" == "ntfs" ] || [ "$fstype" == "vfat" ] || [ "$fstype" == "ext4" ] || [ "$fstype" == "btrfs" ]; then
		devcount=$(($devcount+1))
		lastdev="$dev"
		menulist="$menulist \"$dev\" \"файловая система: $fstype\""
	fi
done < <(lsblk -nro NAME,FSTYPE "/dev/$dstdev")

if [ $devcount == 0 ]; then # No file systems found
	dialog --msgbox "Доступных файловых систем на накопителе \"$dstdev\" не обнаружено! Обратите внимание, что в настоящее время программой \"altquire\" поддерживаются только файловые системы NTFS, FAT, Ext4 и btrfs." 12 60
	rm -f "$descrfile" 2> /dev/null
	exec ./4-select-mode.sh "$srcdev" "$dstdev"
else # File system(s) found

	# Display menulist
	tmpfile=`mktemp 2> /dev/null`
	command="dialog --visit-items --cancel-label \"В начало\" --ok-label \"Выбрать файловую систему\" --menu \"Выберите файловую систему, в которую будут скопированы данные:\" 23 75 20 $menulist 2> "$tmpfile""
	eval "$command"
	ec="$?"
	mountdevice=`cat "$tmpfile" 2> /dev/null`
	rm -f "$tmpfile" 2> /dev/null

	# What did user select?
	case $ec in
		255) exit 255;; # Error or "ESC" key pressed
		0) # "OK" button pressed
				if [ ! -z "$mountdevice" ]; then
					fs=`blkid -o value -s TYPE "/dev/$mountdevice" 2> /dev/null`
					mountdev_and_prepare "$mountdevice"
					if [ ! -z "$dstdir" ]; then
						exec ./128-dc3dd-file.sh "/dev/$srcdev" "$dstdir" "$fs" "$mountpoint" # Next step
					fi
				else
					exec $0 "$srcdev" "$dstdev" "$descrfile" # Nothing was selected
				fi
			;;
		1) # "Cancel" button pressed
			rm -f "$descrfile" 2> /dev/null
			exec ./1-select-source.sh
			;;
	esac
fi
